"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function AbilityService(app) {
    app.get('/api/ability/get_abilitys/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AbilityService::get_abilitys] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilitys(instance, search_criteria, search_expression, disabled);
        }).then(function (abilitys) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: abilitys });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/create_ability', function (req, res, next) {
        logger.info('[AbilityService::create_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newAbility = (req.body || {});
        newAbility.createdDate = nowUtc;
        newAbility.createdByUserId = userId;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        newAbility.title = newAbility.title.trim();
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilityTitle(instance, newAbility.title);
        }).then(function (result) {
            if (result)
                throw 'Ya existe una habilidad con la descripción \'' + newAbility.title + '\'';
            newAbility.id = utilities_1.guid();
            return SaveAbility(instance, newAbility);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al guardar la habilidad';
            return SaveScore(instance, newAbility);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al guardar la habilidad';
            return Promise.resolve(newAbility.id);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'OK' : 'Abilidad save failed'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AbilityService::create_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_ability/:ability_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var ability = {};
        var scores = [];
        var abilityId = req.params.ability_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!abilityId)
                throw 'Missing ability input parameter';
            return mssql_service_1.MSSql.GetAbility(instance, abilityId);
        }).then(function (result) {
            if (!abilityId)
                throw 'Missing ability input parameter';
            ability = result || ability;
            return mssql_service_1.MSSql.GetScores(instance, abilityId);
        }).then(function (resultScores) {
            if (res.headersSent)
                return;
            scores = resultScores || scores;
            ability.scores = scores;
            res.json({ status: true, description: 'OK', data: ability });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/save_ability', function (req, res, next) {
        logger.info('[AbilityService::save_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = req.app.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newAbility = (req.body || {});
        newAbility.createdDate = null;
        newAbility.createdByUserId = null;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilityTitle(instance, newAbility.title);
        }).then(function (result) {
            var abilityDesc = {};
            abilityDesc = result || abilityDesc;
            if (result && abilityDesc.id !== newAbility.id)
                throw 'Ya existe una habilidad con la descripción \'' + newAbility.title + '\'';
            return mssql_service_1.MSSql.SaveAbility(instance, newAbility.id, newAbility.title, newAbility.detail, newAbility.type, newAbility.binary, newAbility.notApply, newAbility.acceptComments, newAbility.createdDate, newAbility.createdByUserId, newAbility.lastModifiedDate, newAbility.lastModifiedByUserId, newAbility.enabled, newAbility.query, newAbility.queryName);
        }).then(function (resultIdAbility) {
            return mssql_service_1.MSSql.DeleteScore(instance, newAbility.id);
        }).then(function (resultDeleteScore) {
            return SaveScore(instance, newAbility);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'OK' : 'Ability save failed'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AbilityService::save_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_ability', function (req, res, next) {
        logger.info('[AbilityService::delete_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var ability = (req.body || {});
        var abilityId = ability.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!ability)
                throw 'Missing ability input parameter';
            return mssql_service_1.MSSql.DeleteScore(instance, abilityId);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteAbility(instance, abilityId);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al eliminar ability';
            return Promise.resolve(true);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'OK' : 'Vcc delete failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AbilityService::delete_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_abilitys', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var abilitys = req.body.abilitys || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!abilitys)
                throw 'Missing vccs input parameter';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteScore(instance, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!abilitys)
                throw 'Missing vccs input parameter';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAbility(instance, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteVccs) {
            var result;
            if (resultDeleteVccs.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Vccs eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteVccs) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteVccs, description: (resultDeleteVccs ? 'OK' : 'Vccs delete failed'), data: resultDeleteVccs });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AbilityService = AbilityService;
function SaveAbility(instance, ability) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAbility(instance, ability.id, ability.title, ability.detail, ability.type, ability.binary, ability.notApply, ability.acceptComments, ability.createdDate, ability.createdByUserId, ability.lastModifiedDate, ability.lastModifiedByUserId, ability.enabled, ability.query, ability.queryName));
    return Promise.all(promiseArray).then(function (results) {
        var falseResults = results.filter(function (res) { return res[0].result !== true && res[0].result !== 1; });
        if (falseResults.length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveScore(instance, ability) {
    var promiseArray = [];
    if (ability.scores) {
        ability.scores.forEach(function (score) {
            promiseArray.push(mssql_service_1.MSSql.SaveScore(instance, score.id, ability.id, score.description, score.value));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
