"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var mssql_service_1 = require("../db/mssql.service");
var integration_config_1 = require("../config/integration.config");
var mssql_config_1 = require("../config/mssql.config");
var http_request_occ_1 = require("../shared/http-request-occ");
var logger = log4js.getLogger('ServerScripts');
function AgentsService(app) {
    app.get('/api/agent/get_agents?', function (req, res, next) {
        logger.info('[AgentsService::get_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var agents = [];
        var agentStr = req.query.agents || '';
        var campaign = req.query.campaign || null;
        logger.info('[AgentsService::get_agents] Received vcc::agents::campaigns => ' + vcc, '::', agentStr, '::', campaign);
        var agentRole = 'quality_agent';
        var isAgent = res.locals.userRole.id === agentRole;
        if (isAgent) {
            logger.info('[AgentsService::get_agents] Agent role quality_agent.');
            agentStr = res.locals.userId;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagAllegro, campaign));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaign));
                    break;
                case 'OCC':
                    var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagOCC, campaign));
                    promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaign));
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_AGENTS_FAILED';
                    agents = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
                    agents = result[0].body;
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_AGENTS_FAILED';
                    var allegroAgents = result[0] ? result[0] : [];
                    var agentsApi = void 0;
                    if (result[1] && result[1].status === true) {
                        agentsApi = result[1] && result[1].status ? result[1].body : [];
                        agents = allegroAgents.concat(agentsApi);
                    }
                    else {
                        agents = allegroAgents;
                    }
                    break;
                case 'OTHER':
                    break;
            }
            if (isAgent)
                agents = agents.filter(function (a) { return a.userId === agentStr; });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: agents });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/agent/evaluation', function (req, res, next) {
        logger.info('[AgentsService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var referrer = res.locals.pageReferrer.split('/').pop();
        var searchType = null;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAgentsFromEvaluation(vcc, userId, campaigns, searchType);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AGENTS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AgentsService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/agent/get_agent_qualification_average/:agentId', function (req, res, next) {
        logger.info('[AgentsService::get_agent_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var agentStr = req.params.agentId || '';
        logger.info('[AgentsService::get_agent_qualification_average] Received instance ' + instance);
        logger.info('[AgentsService::get_agent_qualification_average] Received agent ' + agentStr);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[AgentsService::get_agent_qualification_average] Calling GetAgentQualificationAverage SP');
            return mssql_service_1.MSSql.GetAgentQualificationAverage(instance, agentStr);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.debug('[AgentsService::get_agent_qualification_average] Received result from GetAgentQualificationAverage SP: ' + JSON.stringify(result));
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[AgentsService::get_agent_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AgentsService = AgentsService;
