"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rp = require("request-promise-native");
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function EnvironmentService(app) {
    app.post('/api/environment/get_environments/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EnvironmentService::get_environments] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var search_allegro = req.body.includeAllegro !== undefined ? req.body.includeAllegro : true;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironments(vcc, search_criteria, search_expression, search_allegro);
        }).then(function (resultEnvironments) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultEnvironments });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environmentId = req.params.environment_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!environmentId)
                throw 'Missing environmentId input parameter';
            return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
        }).then(function (resultEnvironment) {
            if (res.headersSent)
                return;
            resultEnvironment.dbPassword = '******';
            resultEnvironment.pbxPassword = '******';
            res.json({ status: true, description: 'OK', data: resultEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment_type_by_name/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environmentId = req.params.environment_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!environmentId)
                throw 'Missing environmentId input parameter';
            return mssql_service_1.MSSql.GetEnvironmentTypeByName(vcc, environmentId);
        }).then(function (resultEnvironmentType) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultEnvironmentType });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment_vcc_list/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironmentVccs(vcc, search_criteria, search_expression);
        }).then(function (resultEnvironmentVccs) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultEnvironmentVccs });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/create_environment', function (req, res, next) {
        logger.info('[EnvironmentService::create_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEnvironment = (req.body || {});
        newEnvironment.apiUser = newEnvironment.apiUser.trim();
        newEnvironment.apiPassword = newEnvironment.apiPassword.trim();
        newEnvironment.apiHost = newEnvironment.apiHost.trim();
        newEnvironment.apiPort = newEnvironment.apiPort;
        newEnvironment.createdDate = nowUtc;
        newEnvironment.createdByUserId = userId;
        newEnvironment.lastModifiedDate = nowUtc;
        newEnvironment.lastModifiedByUserId = userId;
        newEnvironment.dbDisabled = (newEnvironment.dbDisabled ? newEnvironment.dbDisabled : false);
        logger.info('[EnvironmentService::create_environment] Params: ', newEnvironment);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEnvironmentByName(vcc, newEnvironment.name);
        }).then(function (resultEnvironment) {
            if (resultEnvironment)
                throw 'Ya existe un ambiente con el nombre \'' + newEnvironment.name + '\'';
            return SaveEnvironment(vcc, newEnvironment);
        }).then(function (resultSaveEnvironment) {
            return Promise.resolve(resultSaveEnvironment);
        }).then(function (resultSaveEnvironment) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveEnvironment ? 'OK' : 'Environment save failed'), data: resultSaveEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[EnvironmentService::create_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/environment/save_environment', function (req, res, next) {
        logger.info('[EnvironmentService::save_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var environment = (req.body || {});
        environment.apiUser = environment.apiUser.trim();
        environment.apiPassword = environment.apiPassword.trim();
        environment.apiHost = environment.apiHost.trim();
        environment.apiPort = environment.apiPort;
        environment.createdDate = null;
        environment.createdByUserId = null;
        environment.lastModifiedDate = nowUtc;
        environment.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (resultDeleteEnvironment) {
            if (!resultDeleteEnvironment)
                throw 'Error al ejecutar solicitud';
            return SaveEnvironment(vcc, environment);
        }).then(function (resultSaveEnvironment) {
            if (!resultSaveEnvironment)
                return false;
            return Promise.resolve(true);
        }).then(function (resultSaveEnvironment) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveEnvironment, description: (resultSaveEnvironment ? 'OK' : 'Environment save failed'), data: resultSaveEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[EnvironmentService::save_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environment', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environment = req.body.environment;
        var environmentId = environment.name;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!environment)
                throw 'Missing environment input parameter';
            return mssql_service_1.MSSql.DeleteEnvironment(vcc, environmentId);
        }).then(function (resultDeleteEnvironment) {
            if (!resultDeleteEnvironment)
                throw 'Se produjo un error al eliminar el ambiente';
            return Promise.resolve(true);
        }).then(function (resultDeleteEnvironment) {
            if (res.headersSent)
                return;
            if (resultDeleteEnvironment)
                logger.info('Ambiente \'' + environmentId + '\' eliminado correctamente');
            res.json({ status: resultDeleteEnvironment, description: (resultDeleteEnvironment ? 'OK' : 'Environment delete failed'), data: resultDeleteEnvironment });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environments', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environments = req.body.environments || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!environments)
                throw 'Missing environments input parameter';
            var promiseArray = [];
            environments.forEach(function (environment) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEnvironment(vcc, environment.name));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteEnvironments) {
            var result;
            if (resultDeleteEnvironments.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Ambientes eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteEnvironments) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteEnvironments, description: (resultDeleteEnvironments ? 'OK' : 'Environments delete failed'), data: resultDeleteEnvironments });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/allegro/ping_environment/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var environmentId = req.params.environment_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!environmentId)
                throw 'Missing environment input parameter';
            return GetEnvironment(vcc, environmentId);
        }).then(function (resultEnvironment) {
            return PingAllegroEnvironment(resultEnvironment.apiHost, resultEnvironment.apiPort);
        }).then(function (response) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: response });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/allegro/get_vcc_list/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (resultEnvironment) {
            return mssql_service_1.MSSql.GetAllegroVccs(vcc, search_criteria, search_expression);
        }).then(function (vccList) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: vccList });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/allegro/get_bulk_vcc/:environment_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var environmentId = req.params.environment_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return GetEnvironment(vcc, environmentId);
        }).then(function (resultEnvironment) {
            return GetBulkVcc(vcc, resultEnvironment, userId);
        }).then(function (resultSaveBulkVcc) {
            if (!resultSaveBulkVcc)
                return false;
            return Promise.resolve(true);
        }).then(function (resultSaveBulkVcc) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveBulkVcc, description: (resultSaveBulkVcc ? 'OK' : 'BulkVcc save failed'), data: resultSaveBulkVcc });
        }).catch(function (err) {
            var environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
}
exports.EnvironmentService = EnvironmentService;
function GetEnvironment(vcc, id) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetEnvironment(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetEnvironment: ' + errMsg);
    });
}
function GetEnvironmentTypeByName(vcc, id) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetEnvironmentTypeByName(vcc, id);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetEnvironment: ' + errMsg);
    });
}
function PingAllegroEnvironment(ip, port) {
    var options = {
        url: 'http://' + ip + (port === '80' ? '' : ':' + port) + '/inconcert/apps/login/form?app=dashboard',
        method: 'GET',
        json: true,
        strictSSL: false,
        timeout: 5000,
    };
    return rp(options).catch(function (err) {
        if (err && err.message && err.message.includes('ECONNREFUSED')) {
            logger.warn('Se produjo un error de conexión al enviar request hacia \'' + options.url + '\'. Se hará un reintento.');
            return rp(options);
        }
        throw err;
    }).then(function (response) {
        if (response.statusCode === 200) {
            return JSON.parse(response.complete);
        }
        else {
            throw response.message || response.internalCode || '';
        }
    });
}
exports.PingAllegroEnvironment = PingAllegroEnvironment;
function SaveEnvironment(vcc, environment) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEnvironments(vcc, environments) {
    var promiseArray = [];
    if (environments) {
        environments.forEach(function (environment) {
            promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveEnvironments = SaveEnvironments;
function GetBulkVcc(vcc, environment, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetAllegroIntegrationVccs(environment);
    }).then(function (resultVccs) {
        return mssql_service_1.MSSql.BulkAllegroVccTemp(vcc, environment, resultVccs, userId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar BulkVcc: ' + errMsg);
    });
}
